/**
* Copyright (c) 2014 - 2017, Nordic Semiconductor ASA
* 
 * All rights reserved.
* 
 * Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
* 
 * 1. Redistributions of source code must retain the above copyright notice, this
*    list of conditions and the following disclaimer.
* 
 * 2. Redistributions in binary form, except as embedded into a Nordic
*    Semiconductor ASA integrated circuit in a product or a software update for
*    such product, must reproduce the above copyright notice, this list of
*    conditions and the following disclaimer in the documentation and/or other
*    materials provided with the distribution.
* 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
* 
 * 4. This software, with or without modification, must only be used with a
*    Nordic Semiconductor ASA integrated circuit.
* 
 * 5. Any software provided in binary form under this license must not be reverse
*    engineered, decompiled, modified and/or disassembled.
* 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
* LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
* OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* 
 */
/** @file
*
* @defgroup ble_sdk_uart_over_ble_main main.c
* @{
* @ingroup  ble_sdk_app_nus_eval
* @brief    UART over BLE application main file.
*
* This file contains the source code for a sample application that uses the Nordic UART service.
* This application uses the @ref srvlib_conn_params module.
*/



#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "ble_hci.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_params.h"
#include "nrf_sdh.h"
#include "nrf_sdh_soc.h"
#include "nrf_sdh_ble.h"
#include "nrf_ble_gatt.h"
#include "app_timer.h"
#include "ble_nus.h"
#include "app_uart.h"
#include "app_util_platform.h"
#include "bsp_btn_ble.h"
//#include "pm_m.h"

#if defined (UART_PRESENT)
#include "nrf_uart.h"
#endif
#if defined (UARTE_PRESENT)
#include "nrf_uarte.h"
#endif

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"
#include "nrf_fstorage.h"
#include "nrf_fstorage_sd.h"

/*libraries added for SPI and Timer feature*/
#include "nrf_drv_spi.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "boards.h"
#include "app_error.h"
#include "nrf_drv_gpiote.h"
#include "nrf_drv_timer.h"
#include "bsp.h"
#include "nrf_spi_mngr.h"

#include "nrf_pwr_mgmt.h"

#define APP_BLE_CONN_CFG_TAG            1                                           /**< A tag identifying the SoftDevice BLE configuration. */

#define APP_FEATURE_NOT_SUPPORTED       BLE_GATT_STATUS_ATTERR_APP_BEGIN + 2        /**< Reply when unsupported features are requested. */

#define DEVICE_NAME                     "Thermo Digital Sensor"                               /**< Name of device. Will be included in the advertising data. */
#define NUS_SERVICE_UUID_TYPE           BLE_UUID_TYPE_VENDOR_BEGIN                  /**< UUID type for the Nordic UART Service (vendor specific). */

#define APP_BLE_OBSERVER_PRIO           3                                           /**< Application's BLE observer priority. You shouldn't need to modify this value. */

#define APP_ADV_INTERVAL                64                                     /**< The advertising interval (in units of 0.625 ms. This value corresponds to 250 ms) */
#define APP_ADV_TIMEOUT_IN_SECONDS      60                                         /**< The advertising timeout (in units of seconds). */

#define MIN_CONN_INTERVAL               MSEC_TO_UNITS(20, UNIT_1_25_MS)             /**< Minimum acceptable connection interval (20 ms), Connection interval uses 1.25 ms units. */
#define MAX_CONN_INTERVAL               MSEC_TO_UNITS(75, UNIT_1_25_MS)             /**< Maximum acceptable connection interval (75 ms), Connection interval uses 1.25 ms units. */
#define SLAVE_LATENCY                   0                                           /**< Slave latency. */
#define CONN_SUP_TIMEOUT                MSEC_TO_UNITS(4000, UNIT_10_MS)             /**< Connection supervisory timeout (4 seconds), Supervision Timeout uses 10 ms units. */
#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(5000)                       /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000)                      /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                          /**< Number of attempts before giving up the connection parameter negotiation. */

#define DEAD_BEEF                       0xDEADBEEF                                  /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

#define UART_TX_BUF_SIZE                256                                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE                256                                         /**< UART RX buffer size. */
#define SPI_INSTANCE                    0                                           /**< SPI instance index. */
#define DRDY_PIN                        30//12                                          /**<DRDY pin */    
#define DPIN                            26                                          /**<Debug Pin for DRDY INTR*/
#define POWER_PIN                       3                                          /**<Debug pin for Timer INTR */
#define QUEUE_LENGTH                    20                                          /**<transaction queue length  */

#define WREN                            0x60                                        /**<set write enable latch */
#define WRDI                            0x40                                        /**<Reset write enable latch */
#define RDSR                            0x50                                        /**<Read Status Register */
#define WRSR                            0x10                                        /**<Write Status Register */
#define READ                            0x30                                        /**<Read Data from Memory Array */
#define WRITE                           0x20                                        /**<Write Data to Memory Array */
#define EEPROM_AD_LOW                   0x0000                                      /**<lowest memory location in eeprom to save data */
#define EEPROM_AD_HIGH                  0xFFFF                                      /**<Highest memory location in eeprom to save data */

#define TEMP_AD_HIGH                   27301                                        /**<Highest temperature AD count value */
#define TEMP_AD_LOW                    1875                                         /**<Lowest temperature AD count value */


#define RX_PIN                         16
#define TX_PIN                         14

#define LED_PIN                        2
#define SYSTEM_PIN                     11
#define I2C_SCL                        20
#define I2C_SDA                        21 
#define MEM_RESET_HOLD                 12 
#define NFCT_PIN9                      9
#define NFCT_PIN10                     10 
BLE_NUS_DEF(m_nus);                                                                 /**< BLE NUS service instance. */
NRF_BLE_GATT_DEF(m_gatt);                                                           /**< GATT module instance. */
BLE_ADVERTISING_DEF(m_advertising);                                                 /**< Advertising module instance. */

APP_TIMER_DEF(app_timer_0);                                                         /**<App timer instance 0 */
APP_TIMER_DEF(app_timer_1);                                                         /**<App timer instance 1 */
NRF_SPI_MNGR_DEF(m_spi_mngr_cb, QUEUE_LENGTH, SPI_INSTANCE);                                  /**<spi manager definition. */

    
static uint16_t   m_conn_handle          = BLE_CONN_HANDLE_INVALID;                 /**< Handle of the current connection. */
static uint16_t   m_ble_nus_max_data_len = BLE_GATT_ATT_MTU_DEFAULT - 3;            /**< Maximum length of data (in bytes) that can be transmitted to the e by the Nordic UART service module. */
static ble_uuid_t m_adv_uuids[]          =                                          /**< Universally unique service identifier. */
{
    {BLE_UUID_NUS_SERVICE, NUS_SERVICE_UUID_TYPE}
};


void spi_event_handle(ret_code_t result, void *p_user_data);
void eeprom_read_handle();
void eeprom_write_handle();
void eeprom_rdsr_handle();
static void fstorage_event_handle(nrf_fstorage_evt_t *p_evt);

NRF_FSTORAGE_DEF(nrf_fstorage_t fstorage) =
{
    /* Set a handler for fstorage events. */
    .evt_handler = fstorage_event_handle,

    /* These below are the boundaries of the flash space assigned to this instance of fstorage.
     * You must set these manually, even at runtime, before nrf_fstorage_init() is called.
     * The function nrf5_flash_end_addr_get() can be used to retrieve the last address on the
     * last page of flash available to write data. */
    .start_addr = 0x3e000,
    .end_addr   = 0x4ffff,
};

static void fstorage_event_handle(nrf_fstorage_evt_t * p_evt)
{
    if (p_evt->result != NRF_SUCCESS)
    {
        NRF_LOG_INFO("--> Event received: ERROR while executing an fstorage operation.");
        return;
    }

    switch (p_evt->id)
    {
        case NRF_FSTORAGE_EVT_WRITE_RESULT:
        {
            NRF_LOG_INFO("--> Event received: wrote %d bytes at address 0x%x.",
                         p_evt->len, p_evt->addr);
        } break;

        case NRF_FSTORAGE_EVT_ERASE_RESULT:
        {
            NRF_LOG_INFO("--> Event received: erased %d page from address 0x%x.",
                         p_evt->len, p_evt->addr);
        } break;

        default:
            break;
    }
}

//configuration for serial eeprom
static nrf_drv_spi_config_t const eeprom_config = 
{
    .sck_pin      = SPI_SCK_PIN,                
    .mosi_pin     = SPI_MOSI_PIN,                
    .miso_pin     = SPI_MISO_PIN,                
    .ss_pin       = SS_EEPROM_PIN,                
    .irq_priority = SPI_DEFAULT_CONFIG_IRQ_PRIORITY,         
    .orc          = 0xFF,                                    
    .frequency    = NRF_DRV_SPI_FREQ_4M,                     
    .mode         = NRF_DRV_SPI_MODE_0,                      
    .bit_order    = NRF_DRV_SPI_BIT_ORDER_MSB_FIRST 
};

static nrf_drv_spi_config_t const adc_config = 
{
    .sck_pin      = SPI_SCK_PIN,                
    .mosi_pin     = SPI_MOSI_PIN,                
    .miso_pin     = SPI_MISO_PIN,                
    .ss_pin       = SPI_SS_PIN,                
    .irq_priority = SPI_DEFAULT_CONFIG_IRQ_PRIORITY,         
    .orc          = 0xFF,                                    
    .frequency    = NRF_DRV_SPI_FREQ_4M,                     
    .mode         = NRF_DRV_SPI_MODE_1,                      
    .bit_order    = NRF_DRV_SPI_BIT_ORDER_MSB_FIRST 
};

static volatile bool newData       = false;                                   /**<flag to detect that data in the receiver buffer is non zero. */
static volatile bool DRDY_level_change   = false;                                   /**<flag to detect a change in DRDY pin level */
static volatile bool timer_expired       = false;                                   /**<Flag to detect when timer expires*/  
//static const nrf_drv_spi_t spi           = NRF_DRV_SPI_INSTANCE(SPI_INSTANCE);      /**< SPI instance. */
const nrf_drv_timer_t TIMER_ADC          = NRF_DRV_TIMER_INSTANCE(1);               /**<Timer Instance*/
const nrf_drv_timer_t POWER_TIMER        = NRF_DRV_TIMER_INSTANCE(2);
static int ADc_Reading              = 0;                                       /**<ADC mV reading */  
static int adcTempRead              = 0;                                       /**<ADC Temperature reading */  
static uint8_t txBuffer[2];                                                         /**Transmission Buffer*/       
static uint8_t rxBuffer[2];                                                         /**<Receiver Buffer*/
static uint16_t bufferLength;
static uint16_t ADC_data_length;
static uint8_t sendADCData[5];
static uint8_t sendADCData1[5];
static uint8_t adc_data[10];
uint8_t incomingData[100]; 
uint32_t timer_ticks;
uint32_t power_ticks;
ret_code_t err_code;
int muxChange = 0;
static int count = 0;
static float fTempADCount = 0;
static float fTemperature = 0;

static uint8_t adc_NOP_buf[2];
static uint8_t adc_mux0_pH_ch_buf[3];
static uint8_t adc_mux0_temp_ch_buf[3];
static uint8_t adc_mux1_buf[3];
static uint8_t adc_sys0_buf[3];
static uint8_t adc_idac0_buf[3];
static uint8_t adc_mux1_offset_buf[3];
static uint8_t adc_mux1_gain_cmd_buf[3];
static uint8_t adc_sysocal_cmd_buf[3];
static uint8_t adc_sysgcal_cmd_buf[3];

static bool writeToMemory = false;
static uint8_t eeprom_read_buf[3];
static uint8_t eeprom_write_buf[5];
static uint8_t eeprom_wren_buf[1];
static uint8_t eeprom_rdsr_buf[1];
uint8_t eeprom_data[50];
bool eeprom_write_ready = false;
static bool calibrationExists = false;
static bool bReadEepromData = false;
static bool bleConnected = false;
static bool bIsReset = true;
static int  nCounter=0;
bool isADCReadingValid = false;
bool bInitPeripherals = false;
static int counter=1;
bool timer2_expired = false;
bool dataSent = false;
bool cycleComplete = false;
static volatile bool SysOff_state = false;
static bool bBondingData = false;
uint8_t EepromData[100] = {0};

  static nrf_spi_mngr_transfer_t const NOP_spi_cmd[] = 
  {
    NRF_SPI_MNGR_TRANSFER(adc_NOP_buf, sizeof(adc_NOP_buf), rxBuffer, sizeof(rxBuffer))
  };

  static nrf_spi_mngr_transfer_t const adc_config_reg[] =
  {
    NRF_SPI_MNGR_TRANSFER(adc_mux0_pH_ch_buf, sizeof(adc_mux0_pH_ch_buf), rxBuffer, sizeof(rxBuffer)),
    //NRF_SPI_MNGR_TRANSFER(adc_mux0_temp_ch_buf, sizeof(adc_mux0_temp_ch_buf), rxBuffer, sizeof(rxBuffer)),
    NRF_SPI_MNGR_TRANSFER(adc_mux1_buf, sizeof(adc_mux1_buf), rxBuffer, sizeof(rxBuffer)),
    NRF_SPI_MNGR_TRANSFER(adc_sys0_buf, sizeof(adc_sys0_buf), rxBuffer, sizeof(rxBuffer)),
    NRF_SPI_MNGR_TRANSFER(adc_idac0_buf, sizeof(adc_idac0_buf), rxBuffer, sizeof(rxBuffer))
  };

  static nrf_spi_mngr_transfer_t const adc_pH_channel[] =
  {
    NRF_SPI_MNGR_TRANSFER(adc_mux0_pH_ch_buf, sizeof(adc_mux0_pH_ch_buf), rxBuffer, sizeof(rxBuffer))
  };

  static nrf_spi_mngr_transfer_t const adc_temp_channel[] =
  {
    NRF_SPI_MNGR_TRANSFER(adc_mux0_temp_ch_buf, sizeof(adc_mux0_temp_ch_buf), rxBuffer, sizeof(rxBuffer))
  };

  static nrf_spi_mngr_transfer_t const adc_mux_normal_cmd[] =
  {
    NRF_SPI_MNGR_TRANSFER(adc_mux1_buf, sizeof(adc_mux1_buf), rxBuffer, sizeof(rxBuffer))
  };

  static nrf_spi_mngr_transfer_t const adc_sysocal_cmd[] = 
  {
    NRF_SPI_MNGR_TRANSFER(adc_mux1_offset_buf, sizeof(adc_mux1_offset_buf), rxBuffer, sizeof(rxBuffer)),
    NRF_SPI_MNGR_TRANSFER(adc_sysocal_cmd_buf, sizeof(adc_sysocal_cmd_buf), rxBuffer, sizeof(rxBuffer))
  };

  static nrf_spi_mngr_transfer_t const adc_sysgcal_cmd[] = 
  {
    NRF_SPI_MNGR_TRANSFER(adc_mux1_gain_cmd_buf, sizeof(adc_mux1_gain_cmd_buf), rxBuffer, sizeof(rxBuffer)),
    NRF_SPI_MNGR_TRANSFER(adc_sysgcal_cmd_buf, sizeof(adc_sysgcal_cmd_buf), rxBuffer, sizeof(rxBuffer))
  };

  static nrf_spi_mngr_transfer_t const eeprom_read_cmd[] =
  {
    NRF_SPI_MNGR_TRANSFER(eeprom_read_buf, sizeof(eeprom_read_buf), rxBuffer, sizeof(rxBuffer))
  };

  static nrf_spi_mngr_transfer_t const eeprom_write_cmd[] =
  {
    NRF_SPI_MNGR_TRANSFER(eeprom_write_buf, sizeof(eeprom_write_buf), rxBuffer, sizeof(rxBuffer))
  };

  static nrf_spi_mngr_transfer_t const eeprom_wren_cmd[] =
  {
    NRF_SPI_MNGR_TRANSFER(eeprom_wren_buf, sizeof(eeprom_wren_buf), rxBuffer, sizeof(rxBuffer))
  };
    static nrf_spi_mngr_transfer_t const eeprom_rdsr_cmd[] =
  {
    NRF_SPI_MNGR_TRANSFER(eeprom_rdsr_buf, sizeof(eeprom_rdsr_buf), rxBuffer, sizeof(rxBuffer))
  };

  static nrf_spi_mngr_transaction_t ph_channel_config = 
  {
    .begin_callback       = NULL,
    .end_callback         = NULL,
    .p_user_data          = rxBuffer,
    .p_transfers          = adc_pH_channel,
    .number_of_transfers  = sizeof(adc_pH_channel)/sizeof(adc_pH_channel[0]),
    .p_required_spi_cfg   = &adc_config
  };

  static nrf_spi_mngr_transaction_t temp_channel_config = 
  {
    .begin_callback       = NULL,
    .end_callback         = NULL,
    .p_user_data          = rxBuffer,
    .p_transfers          = adc_temp_channel,
    .number_of_transfers  = sizeof(adc_temp_channel)/sizeof(adc_temp_channel[0]),
    .p_required_spi_cfg   = &adc_config
  };

  static nrf_spi_mngr_transaction_t eeprom_transaction[] = 
  {
    {
      .begin_callback       = NULL,
      .end_callback         = eeprom_read_handle,
      .p_user_data          = rxBuffer,
      .p_transfers          = eeprom_read_cmd,
      .number_of_transfers  = sizeof(eeprom_read_cmd)/sizeof(eeprom_read_cmd[0]),
      .p_required_spi_cfg   = &eeprom_config
    },
    {
      .begin_callback       = NULL,
      .end_callback         = NULL,
      .p_user_data          = rxBuffer,
      .p_transfers          = eeprom_wren_cmd,
      .number_of_transfers  = 1,
      .p_required_spi_cfg   = &eeprom_config
    },
    {
      .begin_callback       = NULL,
      .end_callback         = eeprom_rdsr_handle,
      .p_user_data          = rxBuffer,
      .p_transfers          = eeprom_rdsr_cmd,
      .number_of_transfers  = 1,
      .p_required_spi_cfg   = &eeprom_config
    },
    {
      .begin_callback       = NULL,
      .end_callback         = eeprom_write_handle,
      .p_user_data          = rxBuffer,
      .p_transfers          = eeprom_write_cmd,
      .number_of_transfers  = sizeof(eeprom_write_cmd)/sizeof(eeprom_write_cmd[0]),
      .p_required_spi_cfg   = &eeprom_config
    },
  };

  static nrf_spi_mngr_transaction_t adc_transaction = 
  {
    .begin_callback       = NULL,
    .end_callback         = spi_event_handle,
    .p_user_data          = rxBuffer,
    .p_transfers          = NOP_spi_cmd,
    .number_of_transfers  = sizeof(NOP_spi_cmd)/sizeof(NOP_spi_cmd[0]),
    .p_required_spi_cfg   = &adc_config
  };

  static nrf_spi_mngr_transaction_t adc_configuration[] = 
  {
    {
      .begin_callback       = NULL,
      .end_callback         = spi_event_handle,
      .p_user_data          = rxBuffer,
      .p_transfers          = &adc_config_reg[0],
      .number_of_transfers  = 1,
      .p_required_spi_cfg   = &adc_config
    },
    {
      .begin_callback       = NULL,
      .end_callback         = spi_event_handle,
      .p_user_data          = rxBuffer,
      .p_transfers          = &adc_config_reg[1],
      .number_of_transfers  = 1,
      .p_required_spi_cfg   = &adc_config
    },
    {
      .begin_callback       = NULL,
      .end_callback         = spi_event_handle,
      .p_user_data          = rxBuffer,
      .p_transfers          = &adc_config_reg[2],
      .number_of_transfers  = 1,
      .p_required_spi_cfg   = &adc_config
    },
    {
      .begin_callback       = NULL,
      .end_callback         = spi_event_handle,
      .p_user_data          = rxBuffer,
      .p_transfers          = &adc_config_reg[3],
      .number_of_transfers  = 1,
      .p_required_spi_cfg   = &adc_config
    },
  };

  static nrf_spi_mngr_transaction_t adc_system_offset_config = 
  {
    .begin_callback       = NULL,
    .end_callback         = spi_event_handle,
    .p_user_data          = rxBuffer,
    .p_transfers          = adc_sysocal_cmd,
    .number_of_transfers  = sizeof(adc_sysocal_cmd)/sizeof(adc_sysocal_cmd[0]),
    .p_required_spi_cfg   = &adc_config
  };

  static nrf_spi_mngr_transaction_t adc_system_gain_config = 
  {
    .begin_callback       = NULL,
    .end_callback         = spi_event_handle,
    .p_user_data          = rxBuffer,
    .p_transfers          = adc_sysgcal_cmd,
    .number_of_transfers  = sizeof(adc_sysgcal_cmd)/sizeof(adc_sysgcal_cmd[0]),
    .p_required_spi_cfg   = &adc_config
  };

  static nrf_spi_mngr_transaction_t adc_mux_normal = 
  {
    .begin_callback       = NULL,
    .end_callback         = spi_event_handle,
    .p_user_data          = rxBuffer,
    .p_transfers          = adc_mux_normal_cmd,
    .number_of_transfers  = sizeof(adc_mux_normal_cmd)/sizeof(adc_mux_normal_cmd[0]),
    .p_required_spi_cfg   = &adc_config
  };

const int TempAdcCurve_30K[] = {
                                  27301,27063,26821,26570,26313,26053,25786,25513,25235,24952,  // -10 to -1       
                                  24664,24367,24065,23759,23448,23134,22815,22494,22168,21840,         //  0 to  9
                                  21509,21173,20835,20495,20154,19811,19469,19124,18781,18438,  // 10 to 19
                                  18096,17751,17406,17064,16724,16384,16047,15710,15378,15049,  // 20 to 29
                                  14721,14397,14075,13757,13443,13135,12828,12528,12232,11941,  // 30 to 39
                                  11655,11370,11091,10820,10552,10288,10028,9778,9528,9290,     // 40 to 49
                                  9057, 8815, 8591, 8369, 8148, 7935, 7727, 7525,7327,7134,  // 50 to 59
                                  6946, 6762, 6582, 6406, 6235, 6069, 5907, 5750,5596,5447,  // 60 to 69
                                  5302, 5159, 5020, 4886, 4755, 4627, 4503, 4384,4267,4161,  // 70 to 79
                                  4043, 3935, 3830, 3728, 3630, 3533, 3439, 3350,3261,3174,  // 80 to 89
                                  3091, 3010, 2931, 2854, 2780, 2707, 2637, 2569,2503,2438,  // 90 to 99
                                  2377, 2316, 2259, 2203, 2149, 2098, 2049, 2002,1957,1915,  // 100 to 109 
                                  1875,1835                                                  // 110 to 111
} ;

const int *PtrTable;
                                                        /**<Length of the buffer*/
/**@brief Function for assert macro callback.
*
* @details This function will be called in case of an assert in the SoftDevice.
*
* @warning This handler is an example only and does not fit a final product. You need to analyse
*          how your product is supposed to react in case of Assert.
* @warning On assert from the SoftDevice, the system can only recover on reset.
*
* @param[in] line_num    Line number of the failing ASSERT call.
* @param[in] p_file_name File name of the failing ASSERT call.
*/
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}


/**@brief Function for the GAP initialization.
*
* @details This function will set up all the necessary GAP (Generic Access Profile) parameters of
*          the device. It also sets the permissions and appearance.
*/
static void gap_params_init(void)
{
    uint32_t                err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);

    err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *) DEVICE_NAME,
                                          strlen(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
}

void DRDY_int_disable();
void DRDY_int_enable();
void timerCompare(uint32_t ticks);
void power_down();
void button_INTR_disable();


/**@brief Function for handling the data from the Nordic UART Service.
*
* @details This function will process the data received from the Nordic UART BLE Service and send
*          it to the UART module.
*
* @param[in] p_nus    Nordic UART Service structure.
* @param[in] p_data   Data to be send to UART module.
* @param[in] length   Length of the data.
*/
/**@snippet [Handling the data received over BLE] */
static void nus_data_handler(ble_nus_evt_t * p_evt)
{
    DRDY_int_disable(); //disable DRDY pin interrupt

    if (p_evt->type == BLE_NUS_EVT_RX_DATA)
    {
        uint32_t err_code;
        for (uint32_t i = 0; i < p_evt->params.rx_data.length; i++)
        {
            do
            {
                //err_code = app_uart_put(p_evt->params.rx_data.p_data[i]);
                if(p_evt->params.rx_data.length > 10){
                  incomingData[i] = p_evt->params.rx_data.p_data[i];
                }
                if ((err_code != NRF_SUCCESS) && (err_code != NRF_ERROR_BUSY))
                {
                    NRF_LOG_ERROR("Failed receiving NUS message. Error 0x%x. ", err_code);
                    APP_ERROR_CHECK(err_code);
                }
            } while (err_code == NRF_ERROR_BUSY);
        }
        if (p_evt->params.rx_data.p_data[p_evt->params.rx_data.length-1] == '\r')
        {
            //while (app_uart_put('\n') == NRF_ERROR_BUSY);
        }
        if(p_evt->params.rx_data.length > 10)
        {
          calibrationExists = true;
        }

       // eepromWrite(&incomingData);
    }
    DRDY_int_enable();

}
/**@snippet [Handling the data received over BLE] */


/**@brief Function for initializing services that will be used by the application.
*/
static void services_init(void)
{
    uint32_t       err_code;
    ble_nus_init_t nus_init;

    memset(&nus_init, 0, sizeof(nus_init));

    nus_init.data_handler = nus_data_handler;

    err_code = ble_nus_init(&m_nus, &nus_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling an event from the Connection Parameters Module.
*
* @details This function will be called for all events in the Connection Parameters Module
*          which are passed to the application.
*
* @note All this function does is to disconnect. This could have been done by simply setting
*       the disconnect_on_fail config parameter, but instead we use the event handler
*       mechanism to demonstrate its use.
*
* @param[in] p_evt  Event received from the Connection Parameters Module.
*/
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    uint32_t err_code;

    if (p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED)
    {
        err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
        APP_ERROR_CHECK(err_code);

    }
}


/**@brief Function for handling errors from the Connection Parameters module.
*
* @param[in] nrf_error  Error code containing information about what went wrong.
*/
static void conn_params_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for initializing the Connection Parameters module.
*/
static void conn_params_init(void)
{
    uint32_t               err_code;
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;

    err_code = ble_conn_params_init(&cp_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for putting the chip into sleep mode.
*
* @note This function will not return.
*/
static void sleep_mode_enter(void)
{
    uint32_t err_code = bsp_indication_set(BSP_INDICATE_IDLE);
    APP_ERROR_CHECK(err_code);

    // Prepare wakeup buttons.
    err_code = bsp_btn_ble_sleep_mode_prepare();
    APP_ERROR_CHECK(err_code);

    // Go to system-off mode (this function will not return; wakeup will cause a reset).
    err_code = sd_power_system_off();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling advertising events.
*
* @details This function will be called for advertising events which are passed to the application.
*
* @param[in] ble_adv_evt  Advertising event.
*/
static void on_adv_evt(ble_adv_evt_t ble_adv_evt)
{
    uint32_t err_code;

    switch (ble_adv_evt)
    {
        case BLE_ADV_EVT_FAST:
            //err_code = bsp_indication_set(BSP_INDICATE_ADVERTISING);
            //APP_ERROR_CHECK(err_code);
            break;
        case BLE_ADV_EVT_IDLE:
            //sleep_mode_enter();
            button_INTR_disable();
            nrf_gpio_cfg_sense_input(SYSTEM_PIN, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_SENSE_HIGH);
            DRDY_int_disable();
            nrf_drv_timer_disable(&POWER_TIMER);
            nrf_gpio_pin_set(LED_PIN);
            nrf_delay_ms(1000);
            nrf_gpio_pin_clear(LED_PIN);
            power_down();
            nrf_delay_ms(1000);
            NRF_POWER->SYSTEMOFF = 1;              
            break;
        default:
            break;
    }
}


/**@brief Function for handling BLE events.
*
* @param[in]   p_ble_evt   Bluetooth stack event.
* @param[in]   p_context   Unused.
*/
static void ble_evt_handler(ble_evt_t const * p_ble_evt, void * p_context)
{
    uint32_t err_code;

    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            NRF_LOG_INFO("Connected");
            //err_code = bsp_indication_set(BSP_INDICATE_CONNECTED);
            //APP_ERROR_CHECK(err_code);
            m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
            bleConnected = true;
            bReadEepromData = true;
            bInitPeripherals = true;
            if(bBondingData)
            {
              bBondingData = false;  
              err_code = sd_ble_gatts_sys_attr_set(m_conn_handle, NULL, 0, 0);
              APP_ERROR_CHECK(err_code);            
            }
            break;

        case BLE_GAP_EVT_DISCONNECTED:
            NRF_LOG_INFO("Disconnected");
            bleConnected = false;
            bBondingData = true;
            // LED indication will be changed when advertising starts.
            m_conn_handle = BLE_CONN_HANDLE_INVALID;
            break;

#ifndef S140
        case BLE_GAP_EVT_PHY_UPDATE_REQUEST:
        {
            NRF_LOG_DEBUG("PHY update request.");
            ble_gap_phys_t const phys =
            {
                .rx_phys = BLE_GAP_PHY_AUTO,
                .tx_phys = BLE_GAP_PHY_AUTO,
            };
            err_code = sd_ble_gap_phy_update(p_ble_evt->evt.gap_evt.conn_handle, &phys);
            APP_ERROR_CHECK(err_code);
        } break;
#endif

        case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
            // Pairing not supported
            err_code = sd_ble_gap_sec_params_reply(m_conn_handle, BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP, NULL, NULL);
            APP_ERROR_CHECK(err_code);
            break;
#if !defined (S112)
         case BLE_GAP_EVT_DATA_LENGTH_UPDATE_REQUEST:
        {
            ble_gap_data_length_params_t dl_params;

            // Clearing the struct will effectivly set members to @ref BLE_GAP_DATA_LENGTH_AUTO
            memset(&dl_params, 0, sizeof(ble_gap_data_length_params_t));
            err_code = sd_ble_gap_data_length_update(p_ble_evt->evt.gap_evt.conn_handle, &dl_params, NULL);
            APP_ERROR_CHECK(err_code);
        } break;
#endif //!defined (S112)
        case BLE_GATTS_EVT_SYS_ATTR_MISSING:
            // No system attributes have been stored.
            err_code = sd_ble_gatts_sys_attr_set(m_conn_handle, NULL, 0, 0);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTC_EVT_TIMEOUT:
            // Disconnect on GATT Client timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gattc_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_TIMEOUT:
            // Disconnect on GATT Server timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gatts_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_EVT_USER_MEM_REQUEST:
            err_code = sd_ble_user_mem_reply(p_ble_evt->evt.gattc_evt.conn_handle, NULL);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_RW_AUTHORIZE_REQUEST:
        {
            ble_gatts_evt_rw_authorize_request_t  req;
            ble_gatts_rw_authorize_reply_params_t auth_reply;

            req = p_ble_evt->evt.gatts_evt.params.authorize_request;

            if (req.type != BLE_GATTS_AUTHORIZE_TYPE_INVALID)
            {
                if ((req.request.write.op == BLE_GATTS_OP_PREP_WRITE_REQ)     ||
                    (req.request.write.op == BLE_GATTS_OP_EXEC_WRITE_REQ_NOW) ||
                    (req.request.write.op == BLE_GATTS_OP_EXEC_WRITE_REQ_CANCEL))
                {
                    if (req.type == BLE_GATTS_AUTHORIZE_TYPE_WRITE)
                    {
                        auth_reply.type = BLE_GATTS_AUTHORIZE_TYPE_WRITE;
                    }
                    else
                    {
                        auth_reply.type = BLE_GATTS_AUTHORIZE_TYPE_READ;
                    }
                    auth_reply.params.write.gatt_status = APP_FEATURE_NOT_SUPPORTED;
                    err_code = sd_ble_gatts_rw_authorize_reply(p_ble_evt->evt.gatts_evt.conn_handle,
                                                               &auth_reply);
                    APP_ERROR_CHECK(err_code);
                }
            }
        } break; // BLE_GATTS_EVT_RW_AUTHORIZE_REQUEST

        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for the SoftDevice initialization.
*
* @details This function initializes the SoftDevice and the BLE event interrupt.
*/
static void ble_stack_init(void)
{
    ret_code_t err_code;

    err_code = nrf_sdh_enable_request();
    APP_ERROR_CHECK(err_code);

    // Configure the BLE stack using the default settings.
    // Fetch the start address of the application RAM.
    uint32_t ram_start = 0;
    err_code = nrf_sdh_ble_default_cfg_set(APP_BLE_CONN_CFG_TAG, &ram_start);
    APP_ERROR_CHECK(err_code);

    // Enable BLE stack.
    err_code = nrf_sdh_ble_enable(&ram_start);
    APP_ERROR_CHECK(err_code);

    // Register a handler for BLE events.
    NRF_SDH_BLE_OBSERVER(m_ble_observer, APP_BLE_OBSERVER_PRIO, ble_evt_handler, NULL);
}


/**@brief Function for handling events from the GATT library. */
void gatt_evt_handler(nrf_ble_gatt_t * p_gatt, nrf_ble_gatt_evt_t const * p_evt)
{
    if ((m_conn_handle == p_evt->conn_handle) && (p_evt->evt_id == NRF_BLE_GATT_EVT_ATT_MTU_UPDATED))
    {
        m_ble_nus_max_data_len = p_evt->params.att_mtu_effective - OPCODE_LENGTH - HANDLE_LENGTH;
        NRF_LOG_INFO("Data len is set to 0x%X(%d)", m_ble_nus_max_data_len, m_ble_nus_max_data_len);
    }
    NRF_LOG_DEBUG("ATT MTU exchange completed. central 0x%x peripheral 0x%x",
                  p_gatt->att_mtu_desired_central,
                  p_gatt->att_mtu_desired_periph);
}


/**@brief Function for initializing the GATT library. */
void gatt_init(void)
{
    ret_code_t err_code;

    err_code = nrf_ble_gatt_init(&m_gatt, gatt_evt_handler);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_ble_gatt_att_mtu_periph_set(&m_gatt, 64);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling events from the BSP module.
*
* @param[in]   event   Event generated by button press.
*/
void bsp_event_handler(bsp_event_t event)
{
    uint32_t err_code;
    switch (event)
    {
        case BSP_EVENT_SLEEP:
            //sleep_mode_enter();
            break;

        case BSP_EVENT_DISCONNECT:
            err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            if (err_code != NRF_ERROR_INVALID_STATE)
            {
                APP_ERROR_CHECK(err_code);
            }
            break;

        case BSP_EVENT_WHITELIST_OFF:
            if (m_conn_handle == BLE_CONN_HANDLE_INVALID)
            {
                err_code = ble_advertising_restart_without_whitelist(&m_advertising);
                if (err_code != NRF_ERROR_INVALID_STATE)
                {
                    APP_ERROR_CHECK(err_code);
                }
            }
            break;

        default:
            break;
    }
}


/**@brief   Function for handling app_uart events.
*
* @details This function will receive a single character from the app_uart module and append it to
*          a string. The string will be be sent over BLE when the last character received was a
*          'new line' '\n' (hex 0x0A) or if the string has reached the maximum data length.
*/
/**@snippet [Handling the data received over UART] */
void uart_event_handle(app_uart_evt_t * p_event)
{
    uint8_t data_array[BLE_NUS_MAX_DATA_LEN];
    static uint8_t index = 0;
    uint32_t       err_code;

    switch (p_event->evt_type)
    {
        case APP_UART_DATA_READY:
            //UNUSED_VARIABLE(app_uart_get(&data_array[index]));
            index++;

            if ((data_array[index - 1] == '\n') || (index >= (m_ble_nus_max_data_len)))
            {
//                NRF_LOG_DEBUG("Ready to send data over BLE NUS");
//                NRF_LOG_HEXDUMP_DEBUG(data_array, index);

                do
                {
                    uint16_t length = (uint16_t)index;
                    err_code = ble_nus_string_send(&m_nus, data_array, &length);
                    if ( (err_code != NRF_ERROR_INVALID_STATE) && (err_code != NRF_ERROR_BUSY) )
                    {
                        APP_ERROR_CHECK(err_code);
                    }
                } while (err_code == NRF_ERROR_BUSY);

                index = 0;
            }
            break;

        case APP_UART_COMMUNICATION_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;

        default:
            break;
    }
}
/**@snippet [Handling the data received over UART] */


/**@brief  Function for initializing the UART module.
*/
/**@snippet [UART Initialization] */
static void uart_init1(void)
{
    uint32_t                     err_code;
    app_uart_comm_params_t const comm_params =
    {
        .rx_pin_no    = RX_PIN,
        .tx_pin_no    = TX_PIN,
        .rts_pin_no   = RTS_PIN_NUMBER,
        .cts_pin_no   = CTS_PIN_NUMBER,
        .flow_control = APP_UART_FLOW_CONTROL_DISABLED,
        .use_parity   = false,
        .baud_rate    = NRF_UART_BAUDRATE_115200
    };

    APP_UART_FIFO_INIT(&comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_event_handle,
                       APP_IRQ_PRIORITY_LOWEST,
                       err_code);
    APP_ERROR_CHECK(err_code);
}
/**@snippet [UART Initialization] */


/**@brief Function for initializing the Advertising functionality.
*/
static void advertising_init(void)
{
    uint32_t               err_code;
    ble_advertising_init_t init;

    memset(&init, 0, sizeof(init));

    init.advdata.name_type          = BLE_ADVDATA_FULL_NAME;
    init.advdata.include_appearance = false;
    init.advdata.flags              = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;//BLE_GAP_ADV_FLAGS_LE_ONLY_LIMITED_DISC_MODE;

    init.srdata.uuids_complete.uuid_cnt = sizeof(m_adv_uuids) / sizeof(m_adv_uuids[0]);
    init.srdata.uuids_complete.p_uuids  = m_adv_uuids;

    init.config.ble_adv_fast_enabled  = true;
    init.config.ble_adv_fast_interval = APP_ADV_INTERVAL;
    init.config.ble_adv_fast_timeout  = APP_ADV_TIMEOUT_IN_SECONDS;

    init.evt_handler = on_adv_evt;

    err_code = ble_advertising_init(&m_advertising, &init);
    APP_ERROR_CHECK(err_code);

    ble_advertising_conn_cfg_tag_set(&m_advertising, APP_BLE_CONN_CFG_TAG);
}


/**@brief Function for initializing buttons and leds.
*
* @param[out] p_erase_bonds  Will be true if the clear bonding button was pressed to wake the application up.
*/
static void buttons_leds_init(bool * p_erase_bonds)
{
    bsp_event_t startup_event;

    uint32_t err_code = bsp_init(BSP_INIT_LED | BSP_INIT_BUTTONS, bsp_event_handler);
    APP_ERROR_CHECK(err_code);

    err_code = bsp_btn_ble_init(NULL, &startup_event);
    APP_ERROR_CHECK(err_code);

    *p_erase_bonds = (startup_event == BSP_EVENT_CLEAR_BONDING_DATA);
}


/**@brief Function for initializing the nrf log module.
*/
static void log_init(void)
{
    ret_code_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();
}


/**@brief Function for placing the application in low power state while waiting for events.
*/
static void power_manage(void)
{
    uint32_t err_code = sd_app_evt_wait();
    APP_ERROR_CHECK(err_code);
}


/**@brief   Helper function to obtain the last address on the last page of the on-chip flash that
*          can be used to write user data.
*/
static uint32_t nrf5_flash_end_addr_get()
{
    uint32_t const bootloader_addr = NRF_UICR->NRFFW[0];
    uint32_t const page_sz         = NRF_FICR->CODEPAGESIZE;
    uint32_t const code_sz         = NRF_FICR->CODESIZE;

    return (bootloader_addr != 0xFFFFFFFF ?
            bootloader_addr : (code_sz * page_sz));
}

double ConvertF2C(double TempF)
{
  return (5.0f* (TempF-32.0f))/ 9.0f ;
}

double ConvertC2F(double TempC)
{
  double dVal=((9.0*TempC)/5.0)+32.0;
  
  return (dVal) ;
}

float CalculateX3(float x1, float x2, float y1, float y2, float y3)
{
  double fltTemp1;
  fltTemp1= ((y3-y1)*(x2-x1)/(y2-y1));
  return (x1+fltTemp1);
}

float GetTemperature(float fTempADCount)
{
    float fTrackTemp = ConvertC2F (-10.0f);             // initialse to start temp, -20 degree C
    
    PtrTable = TempAdcCurve_30K ;       /* initialsie the pointer to std Temp Curve */
        
    
    float   fTempCalculatedC = 25.0f;     /* calculated temperature  */
    float   fTempCalculatedF = 77.0f;     /* calculated temperature  */

    float fWy3 ,fWy2,fWy1,fWx2,fWx1;
    
    fWy3 =  fTempADCount;

    if ( fWy3 < *(PtrTable+121) )               // under range condition
    {
         fTempCalculatedF = -4.1f;       
         fTempCalculatedC = -11.0;  
    }
    else if ( fWy3 > *PtrTable )    // over range condition
    {
         fTempCalculatedF = 249.8f;       
         fTempCalculatedC = 111.0f;       
    }
    else                                  // normal range
    {
        for (int cIndex=0; cIndex < 120; cIndex++)        // adc is in ascending order
        {                                              // table is in ascending order
          if ( *(PtrTable+cIndex) >= fWy3)             // compare the table with the current adc
          { 
              fWy1 = (float) *(PtrTable+cIndex);            // load the adc1 to wy1
              fWx1= fTrackTemp;
              cIndex = cIndex+1;
              fWy2 = (float) *(PtrTable+cIndex);                              // load adc of next one to wy2
              fWx2= fTrackTemp + 1.80;                                        // table in 1 degree C
              fTempCalculatedF = CalculateX3(fWx1,fWx2,fWy1,fWy2,fWy3);       // call the calculation routine to find calculated temp
              fTempCalculatedC= ConvertF2C(fTempCalculatedF);                 // convert to C
              cIndex=121;                                                     // stop the for loop
          }
          fTrackTemp += 1.8;                                                  // for 1 degree C ( 1.8F) jump
        }
    }
    return fTempCalculatedC;
}

void change_mux_channel();

/**@brief Function for handling the timer event when the timer expires
  *@param event
*/
void timer1_event_handler(nrf_timer_event_t event_type, void* p_context)
{  
   //nrf_gpio_pin_toggle(PIN); //uncomment to check for timer trigger

    if(DRDY_level_change)
    {
       DRDY_level_change = false;
      APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &adc_transaction));
      nrf_gpio_pin_clear(EXT_ADC_START);
    }
}

void timer2_event_handle(nrf_timer_event_t event_type, void* p_context)
{
  timer2_expired = true;
}

/**@brief  Function to set the value in CC register and enabling the timer0 instance
  *@params timer ticks
*/
void timerCompare(uint32_t ticks){
  nrf_drv_timer_extended_compare(&TIMER_ADC, NRF_TIMER_CC_CHANNEL0, ticks, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, true);
  nrf_drv_timer_enable(&TIMER_ADC);
}

void powerTimerCompare(uint32_t power_ticks)
{
  nrf_drv_timer_extended_compare(&POWER_TIMER, NRF_TIMER_CC_CHANNEL0, power_ticks, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, true);
  nrf_drv_timer_enable(&POWER_TIMER);  
}

/**@brief Function to handle eeprom read event
*/
void eeprom_read_handle()
{
  static uint8_t byteCount = 0;
  //static uint8_t *pEEpromData;
  if(rxBuffer[0] == 0)
  {
    writeToMemory = true;
    byteCount = 0;
  }
  else
  {
    eeprom_data[byteCount] = rxBuffer[0];
    //*pEEpromData = &eeprom_data[byteCount];
    byteCount++;
    //pEEpromData++;
    if(byteCount>50){byteCount=0;}
  }
}

/**@brief Function to handle eeprom write events 
*/
void eeprom_write_handle()
{
  
}

void eeprom_rdsr_handle()
{
  if((rxBuffer[0] & 0x01) != 0)
  {
    eeprom_write_ready = false;
  }
  else
  {
    eeprom_write_ready = true;
  }
}

/**
* @brief SPI user event handler.
* @param event
*/
void spi_event_handle(ret_code_t result, void *p_user_data)
{
    DRDY_level_change = false;
     if(muxChange == 0)
      {
       if(rxBuffer[0] != 0)
       {
        ADc_Reading=rxBuffer[0];
        ADc_Reading=ADc_Reading<<8;
        ADc_Reading=ADc_Reading|rxBuffer[1];

        if((ADc_Reading & 0x8000) == 0x8000)
        { 
          ADc_Reading = ~(ADc_Reading);
          ADc_Reading = ADc_Reading & (0x00FFFF);
          ADc_Reading += 1;
          ADc_Reading = ADc_Reading*(-1);
        }
        else
        {
          //do nothing
        }
        if(isADCReadingValid)
        {
          itoa(ADc_Reading/16, sendADCData1,10); 
        }
       }
      }
      else if(muxChange == 1)
      { 
       if(rxBuffer[0]!=0)
       {
        adcTempRead=rxBuffer[0];
        adcTempRead=adcTempRead << 8;
        adcTempRead=adcTempRead|rxBuffer[1];
        fTempADCount = (float)adcTempRead;
        fTemperature = GetTemperature(fTempADCount);
        itoa(fTemperature,sendADCData,10);
       }
      }
      else
      {
        //do nothing
      }
//    }
}


/**@brief   Function to handle the DRDY pin level change
   *@params pin, polarity
*/
void DRDY_Handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t NRF_GPIOTE_POLARITY_HITOLO)
{
    DRDY_level_change = true; //new conversion is ready
    newData = true;
    // nrf_gpio_pin_toggle(DPIN); //uncomment to check for DRDY trigger
    //nrf_gpio_pin_clear(EXT_ADC_START);
}

/**@brief Function is to configure and initialize the DRDY pin
*/
void DRDY_pin_init(){

   // nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_TOGGLE(false);
    nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_HITOLO(true);

    err_code = nrf_drv_gpiote_in_init(DRDY_PIN, &in_config, DRDY_Handler);
    APP_ERROR_CHECK(err_code);
}

void button_INTR_disable()
{
  nrf_drv_gpiote_in_event_disable(SYSTEM_PIN);
}

void power_button_handle(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t NRF_GPIOTE_POLARITY_LOTOHI)
{
 
   bIsReset=false;
   NRF_LOG_INFO("interrupt fired!");
     NRF_LOG_FLUSH();
}

void buttonPress()
{
    nrf_drv_gpiote_in_config_t power_button_cfg = GPIOTE_CONFIG_IN_SENSE_LOTOHI(false);
    err_code = nrf_drv_gpiote_in_init(SYSTEM_PIN, &power_button_cfg, power_button_handle);
    APP_ERROR_CHECK(err_code);
}

void button_INTR_EN()
{
  nrf_drv_gpiote_in_event_enable(SYSTEM_PIN, true);
}

/**@brief Function to enable the interrupt on DRDY_PIN
*/
void DRDY_int_enable(){
  nrf_drv_gpiote_in_event_enable(DRDY_PIN, true);
}

/**@brief Function to disable the interrupt on DRDY_PIN
*/
void DRDY_int_disable(){
  nrf_drv_gpiote_in_event_disable(DRDY_PIN);
}

/**@brief  Function to configure the ADC registers
  *@params SPI instance
*/
void ADC_config()
{
    //settings for burn out current source and AIN pins
    adc_mux0_pH_ch_buf[0] = 0x40;//ADC_COMMAND_WRITE_REGISTER(ADC_REG_00_MUX0);
    adc_mux0_pH_ch_buf[1] = 0x00;//ADC_COMMAND_REGISTER_COUNT(3);
    adc_mux0_pH_ch_buf[2] = 0x01;
    //settings for temperature channel pins
    adc_mux0_temp_ch_buf[0] = 0x40;
    adc_mux0_temp_ch_buf[1] = 0x00;
    adc_mux0_temp_ch_buf[2] = 0x13;

    //settings for Internal VREF and initial MUX calibration
    adc_mux1_buf[0] = 0x42;//ADC_COMMAND_WRITE_REGISTER(ADC_REG_02_MUX1);
    adc_mux1_buf[1] = 0x00;//ADC_COMMAND_REGISTER_COUNT(1);
    adc_mux1_buf[2] = 0x30;//ADC_MUX1_VREFCON_IREF_ON;

    //settings for PGA and Data sample rate
    adc_sys0_buf[0] = 0x43;//ADC_COMMAND_WRITE_REGISTER(ADC_REG_03_SYS0);
    adc_sys0_buf[1] = 0x00;//ADC_COMMAND_REGISTER_COUNT(2);
    adc_sys0_buf[2] = 0x01;

    //settings for Data ready mode
    adc_idac0_buf[0] = 0x4A;//ADC_COMMAND_WRITE_REGISTER(ADC_REG_0A_IDAC0);
    adc_idac0_buf[1] = 0x00;//ADC_COMMAND_REGISTER_COUNT(1);
    adc_idac0_buf[2] = 0x00;//ADC_IDAC0_DRDY_MODE0;

    //NOP command
    adc_NOP_buf[0] = 0xFF;
    adc_NOP_buf[1] = 0xFF;
       
    //mux1 register command to set offset
    adc_mux1_offset_buf[0] = 0x42;
    adc_mux1_offset_buf[1] = 0x00;
    adc_mux1_offset_buf[2] = 0x31;

    //mux1 register command to set gain
    adc_mux1_gain_cmd_buf[0] = 0x42;
    adc_mux1_gain_cmd_buf[1] = 0x00;
    adc_mux1_gain_cmd_buf[2] = 0x32;

    //sysocal command for system offset
    adc_sysocal_cmd_buf[0] = 0x62;

    //sysgcal command for system gain
    adc_sysgcal_cmd_buf[0] = 0x61;    
}

void adc_initialize()
{
  APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &adc_configuration[0]));
  nrf_delay_ms(1);
  APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &adc_configuration[1]));
  nrf_delay_ms(5);
  APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &adc_configuration[2]));
  nrf_delay_ms(1);
  APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &adc_configuration[3]));
  nrf_delay_ms(1);
}

void adc_calibrate(nrf_drv_spi_t const * const p_instance_spi)
{

    //settings for system Offset calibration
    memset(rxBuffer,0,bufferLength);
    APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &adc_system_offset_config));
    nrf_delay_ms(1700);

    //settings for Gain calibration
    memset(rxBuffer,0,bufferLength);
    APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &adc_system_gain_config));
    nrf_delay_ms(1700);

    //setting MUX to normal
    memset(rxBuffer,0,bufferLength);
    APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &adc_mux_normal));
    nrf_delay_ms(5);      
}

void change_mux_channel()
{
    nrf_gpio_pin_set(EXT_ADC_START);
    nrf_delay_ms(2); 
    if(muxChange==0)
    {
      muxChange++;
      count++;
      //mux settings for Temperature channel
      APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &temp_channel_config));
      //NRF_LOG_INFO("adc data: %d, temp data=%d", adcTempRead,fTemperature/(1.4));
    }
    else
    {
      muxChange--;
      count++; 
      //mux settings for pH channel 
      APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &ph_channel_config));
      //NRF_LOG_INFO("adc data:%d, pH_mv=%d", ADc_Reading,ADc_Reading/16);
    }
    nrf_delay_ms(5);
    nrf_gpio_pin_clear(EXT_ADC_START);  
}

/**@brief Write Enable Instruction to be used when to program eeprom device
*/
void eepromWREN()
{
  eeprom_wren_buf[0] = WREN;
  APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &eeprom_transaction[1]));  
}

/**@brief Read status Register instruction to check if write to eeprom is done or ongoing
*/
void eepromRDSR()
{
  eeprom_rdsr_buf[0] = RDSR;
  APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &eeprom_transaction[2]));
}

/**@brief Function to write to eeprom
*/
void eepromWrite(uint8_t *calData)
{
  //send write instruction with address bytes and data
  eeprom_write_buf[0] = WRITE;
  eeprom_write_buf[1] = ((EEPROM_AD_LOW & 0xFFFF) >> 8);
  eeprom_write_buf[2] = (EEPROM_AD_LOW & 0xFF);
  for(uint8_t loopCount=0; loopCount<sizeof(calData); loopCount++)
  {
    eeprom_write_buf[3] = calData[loopCount];
    eepromWREN();
    APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &eeprom_transaction[3]));
    //loop to check if write is done/RDSR instruction
    while(!eeprom_write_ready)
    {
      eepromRDSR();
    } 
  } 
}

/**@brief Function to read from eeprom
*/
void eepromRead()
{
  eeprom_read_buf[0] = READ; // read command 0x30
  eeprom_read_buf[1] = ((EEPROM_AD_LOW & 0xFFFF) >> 8); //MSB first
  eeprom_read_buf[2] = (EEPROM_AD_LOW & 0xFF);
  do
  {
    APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &eeprom_transaction[0]));
  }while(!writeToMemory);
}

void power_down()
{
  DRDY_int_disable();
  nrf_gpio_cfg(POWER_PIN, NRF_GPIO_PIN_DIR_OUTPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_S0S1, NRF_GPIO_PIN_NOSENSE);
  nrf_gpio_pin_clear(POWER_PIN);
  nrf_gpio_cfg(EXT_ADC_RESET, NRF_GPIO_PIN_DIR_OUTPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLUP, NRF_GPIO_PIN_S0S1, NRF_GPIO_PIN_NOSENSE);
  nrf_gpio_pin_set(EXT_ADC_RESET);
  nrf_gpio_cfg(EXT_ADC_START, NRF_GPIO_PIN_DIR_OUTPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_S0S1, NRF_GPIO_PIN_NOSENSE);
  nrf_gpio_pin_clear(EXT_ADC_START);
  nrf_gpio_cfg(LED_PIN, NRF_GPIO_PIN_DIR_OUTPUT,NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
  nrf_gpio_pin_clear(LED_PIN);
  nrf_gpio_cfg(DRDY_PIN, NRF_GPIO_PIN_DIR_INPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
  //nrf_gpio_pin_clear(DRDY_PIN);
  nrf_gpio_cfg(SPI_SS_PIN, NRF_GPIO_PIN_DIR_OUTPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLUP, NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
  nrf_gpio_pin_set(SPI_SS_PIN);
  nrf_gpio_cfg(SPI_MOSI_PIN, NRF_GPIO_PIN_DIR_OUTPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
  nrf_gpio_pin_clear(SPI_MOSI_PIN);
  nrf_gpio_cfg(SPI_MISO_PIN, NRF_GPIO_PIN_DIR_INPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
  nrf_gpio_pin_clear(SPI_MISO_PIN);
  nrf_gpio_cfg(SPI_SCK_PIN, NRF_GPIO_PIN_DIR_OUTPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
  nrf_gpio_pin_clear(SPI_SCK_PIN);
  nrf_gpio_cfg(SS_EEPROM_PIN, NRF_GPIO_PIN_DIR_OUTPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLUP, NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
  nrf_gpio_pin_set(SS_EEPROM_PIN);
  nrf_gpio_cfg(MEM_RESET_HOLD, NRF_GPIO_PIN_DIR_INPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLUP, NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
  nrf_gpio_pin_set(MEM_RESET_HOLD);
  nrf_gpio_cfg(RX_PIN, NRF_GPIO_PIN_DIR_INPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLUP, NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
  //nrf_gpio_pin_set(RX_PIN);
  nrf_gpio_cfg(TX_PIN, NRF_GPIO_PIN_DIR_OUTPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLUP, NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
  nrf_gpio_pin_set(TX_PIN);
  nrf_gpio_cfg(I2C_SCL, NRF_GPIO_PIN_DIR_OUTPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLUP, NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
  nrf_gpio_pin_set(I2C_SCL);
  nrf_gpio_cfg(I2C_SDA, NRF_GPIO_PIN_DIR_INPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLUP, NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
  nrf_gpio_pin_set(I2C_SDA);
  nrf_gpio_cfg(NFCT_PIN9, NRF_GPIO_PIN_DIR_INPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
  //nrf_gpio_pin_clear(NFCT_PIN9);
  nrf_gpio_cfg(NFCT_PIN10, NRF_GPIO_PIN_DIR_INPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
  //nrf_gpio_pin_clear(NFCT_PIN10);
}

void pin_config()
{
   nrf_gpio_cfg_sense_input(SYSTEM_PIN, NRF_GPIO_PIN_PULLUP, NRF_GPIO_PIN_SENSE_HIGH);
   nrf_gpio_cfg(LED_PIN, NRF_GPIO_PIN_DIR_OUTPUT,NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_NOPULL,NRF_GPIO_PIN_S0S1,NRF_GPIO_PIN_NOSENSE);
   nrf_gpio_cfg(POWER_PIN, NRF_GPIO_PIN_DIR_OUTPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLUP, NRF_GPIO_PIN_S0S1, NRF_GPIO_PIN_NOSENSE); 
   nrf_gpio_cfg_output(EXT_ADC_START);
   nrf_gpio_pin_set(POWER_PIN);
   DRDY_pin_init(); //initialize DRDY pin
   buttonPress(); //initialize button
   nrf_gpio_cfg_input(NFCT_PIN9,NRF_GPIO_PIN_NOPULL);
   nrf_gpio_cfg_input(NFCT_PIN10,NRF_GPIO_PIN_NOPULL);
   nrf_gpio_pin_clear(NFCT_PIN9);
   nrf_gpio_pin_clear(NFCT_PIN10);
   nrf_gpio_cfg_output(I2C_SCL);
   nrf_gpio_pin_set(I2C_SCL);
   nrf_gpio_cfg_input(I2C_SDA,NRF_GPIO_PIN_NOPULL);
   nrf_gpio_pin_set(I2C_SDA);
   nrf_gpio_cfg_input(MEM_RESET_HOLD,NRF_GPIO_PIN_NOPULL);     
}

/**@brief Application main function.
*/
int main(void)
{
   uint32_t err_code;
    bool     erase_bonds;
    uint32_t pin_sense = 0;
    bIsReset=true;
    counter=1;
    // Initialize.
    err_code = app_timer_init();
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_gpiote_init();
    APP_ERROR_CHECK(err_code);

    pin_config();
    //uart_init1();
    log_init();

    nrf_gpio_pin_set(LED_PIN);
    nrf_delay_ms(1000);
    nrf_gpio_pin_clear(LED_PIN);

    nrf_fstorage_api_t * p_fs_api;
    p_fs_api = &nrf_fstorage_sd;

    err_code = nrf_pwr_mgmt_init();

    //Configure Timer
    nrf_drv_timer_config_t power_timer_cfg = NRF_DRV_TIMER_DEFAULT_CONFIG;
    err_code = nrf_drv_timer_init(&POWER_TIMER, &power_timer_cfg, timer2_event_handle);

    //Configure SPI
    nrf_spi_mngr_init(&m_spi_mngr_cb, &adc_config);   
    APP_ERROR_CHECK(nrf_fstorage_init(&fstorage, p_fs_api, NULL));
    (void)nrf5_flash_end_addr_get();

    //buttons_leds_init(&erase_bonds);
    ble_stack_init();
    gap_params_init();
    gatt_init();
    services_init();
    advertising_init();
    conn_params_init();

   ADC_config();
   nrf_gpio_cfg_output(EXT_ADC_RESET);
   nrf_gpio_pin_set(EXT_ADC_RESET);
   nrf_delay_ms(0.6);
   adc_initialize(); //initialize the adc registers
   nrf_delay_ms(5);
//    nrf_gpio_cfg_output(EXT_ADC_START);
//    nrf_gpio_pin_set(EXT_ADC_START);
//    nrf_delay_ms(2);
    //adc_calibrate(&spi);
    err_code = ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST);
    APP_ERROR_CHECK(err_code);
    power_ticks = nrf_drv_timer_ms_to_ticks(&POWER_TIMER, 2000);
    powerTimerCompare(power_ticks);
//    DRDY_int_enable();
    button_INTR_EN();

   while(true)
    {
      if(bIsReset == false)
      {
        pin_sense = nrf_gpio_pin_read(SYSTEM_PIN);
        if(pin_sense == 1)
        {
          nCounter++;
          if(nCounter > 3000000)
          {
            bIsReset = true;
            nCounter=0;
            bleConnected = false;
            bBondingData = false;
            button_INTR_disable();
            nrf_gpio_cfg_sense_input(SYSTEM_PIN, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_SENSE_HIGH);
            DRDY_int_disable();
            nrf_drv_timer_disable(&POWER_TIMER);
            nrf_gpio_pin_set(LED_PIN);
            nrf_delay_ms(1000);
            nrf_gpio_pin_clear(LED_PIN);
            power_down();
            nrf_delay_ms(1000);
            NRF_POWER->SYSTEMOFF = 1;
          }
        }
      }
      if(pin_sense == 0)
      {
        if(timer2_expired && bleConnected)
        {
          //nrf_gpio_cfg(POWER_PIN, NRF_GPIO_PIN_DIR_OUTPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLUP, NRF_GPIO_PIN_S0S1, NRF_GPIO_PIN_NOSENSE); 
          nrf_gpio_pin_set(POWER_PIN);
//          nrf_gpio_pin_set(LED_PIN);
          nrf_delay_ms(500);
//          nrf_gpio_pin_clear(LED_PIN);
          isADCReadingValid = true;
          nrf_drv_timer_disable(&POWER_TIMER);
          DRDY_int_enable();
          nrf_gpio_pin_set(EXT_ADC_START);
          nrf_delay_ms(2);
          nrf_gpio_pin_clear(EXT_ADC_START);
          nrf_delay_ms(110);
//          if(calibrationExists)
//          {
//            err_code = nrf_fstorage_erase(&fstorage, 0x3e000, 1, NULL);
//            if(err_code == NRF_SUCCESS)
//            {
//              err_code = nrf_fstorage_write(&fstorage, 0x3e000, &incomingData, sizeof(incomingData), NULL); //write to internal flash
//              calibrationExists = false;
//            }
//            else if(err_code == NRF_ERROR_NULL)
//            {
//              calibrationExists = false;
//              NRF_LOG_INFO("instance is null");
//              NRF_LOG_FLUSH();
//            }
//          }
//
//          if(bReadEepromData)
//          {
//            err_code = nrf_fstorage_read(&fstorage, 0x3e000, EepromData, sizeof(incomingData)); //read from internal flash
//          }
          
          if(newData)
          {
            DRDY_int_disable();
            APP_ERROR_CHECK(nrf_spi_mngr_schedule(&m_spi_mngr_cb, &adc_transaction)); //read data from ADC
            nrf_delay_ms(5);
            if(count == 2)
            {
              memcpy(adc_data, sendADCData1, 5);
              strcat(adc_data, ",");
              strcat(adc_data, sendADCData);
              do
              {
                ADC_data_length =10;
  //            uint16_t eeprom_data_length = sizeof(EepromData);
  //            if(bReadEepromData)
  //            {
  //              bReadEepromData = false;
  //              err_code = ble_nus_string_send(&m_nus, EepromData, &eeprom_data_length); //send calibration data over BLE
  //            }
  //            else
  //            {
                  err_code = ble_nus_string_send(&m_nus, adc_data, &ADC_data_length); //send ADC data over BLE
  //            }                        
                if ((err_code != NRF_ERROR_INVALID_STATE) && (err_code != NRF_ERROR_BUSY))
                {
                  APP_ERROR_CHECK(err_code);
                  newData = false;
                  count = 0;
                  timer2_expired = false;
                }
              }while (err_code == NRF_ERROR_BUSY);
            }
            change_mux_channel();
            timer2_expired = false;
            newData = false;
          }
          memset(adc_data,0,10);
//          memset(EepromData, 0, sizeof(EepromData));
            powerTimerCompare(power_ticks);  

        }
        if(bleConnected && !timer2_expired)
        {
          if((counter % 40) == 0)
          {
            nrf_gpio_pin_set(LED_PIN);
            nrf_delay_ms(50);
            nrf_gpio_pin_clear(LED_PIN);
          }      
          counter++;
          if(counter >200)
          {
            counter=1;
          }
          isADCReadingValid = false;
          nrf_gpio_pin_clear(POWER_PIN); 
          UNUSED_RETURN_VALUE(NRF_LOG_PROCESS());
          power_manage();
        }
        else if(!bleConnected)
        {
          UNUSED_RETURN_VALUE(NRF_LOG_PROCESS());        
          power_manage();
        }
        else
        {
          //do nothing
        }
      } 
    }  
 }
/**
* @}
*/
